from django.conf.urls import patterns, url

urlpatterns = patterns('django_social_user.views',
    url(r'^authenticate/(?P<network>\w+)/$', 'authenticate', name='authenticate'),
    url(r'^callback/(?P<network>\w+)/$', 'callback', name='callback'),
)



from logging import getLogger

from django.contrib.auth import authenticate as auth, login
from django.http import Http404
from django.shortcuts import redirect

from django_social_user import registered_networks, options
from django_social_user.exceptions import SocialOauthDictFailed
from django_social_user.signals import (
    django_social_user_pre_auth, django_social_user_post_callback)

logger = getLogger('django_social_user:views')

SK_AUTH_REDIRECT = 'django_social_user:authentication_redirect'
SK_REQUEST_TOKEN = 'django_social_user:oauth_request_token'


def get_network_backend_or_404(network):
    """
    Fetch the network from the registered list or throw a 404 exception.
    """
    backend = registered_networks.get(network)
    if not backend:
        raise Http404
    return backend


def authenticate(request, network):
    """
    Invoked before redirecting the user to a social network. This view
    sets up necessary information for that needed in the callback for
    the oauth exchange.
    """
    backend = get_network_backend_or_404(network)
    django_social_user_pre_auth.send(None, request=request)

    try:
        oauth_request_token = backend.get_oauth_request_token()
        request.session[SK_REQUEST_TOKEN] = oauth_request_token
        return redirect(backend.get_oauth_authorization_url(
            oauth_request_token))
    except:
        logger.exception('Authentication failed for %s' % network)
        r = getattr(options, 'SOCIAL_USER_REDIRECT_ON_REQUEST_TOKEN_FAILURE')
        if r:
            return redirect(r)
        raise


def callback(request, network):
    """
    This is the authentication view called by social network after the
    user authenticates via the social network.
    """
    oauth_request_token = request.session.get(SK_REQUEST_TOKEN)

    if not oauth_request_token:
        raise Http404

    backend = get_network_backend_or_404(network)

    try:
        access_token, access_token_expires = backend.get_oauth_access_token(
            request, oauth_request_token)
    except:
        logger.exception('Callback failed for %s' % network)
        r = getattr(options, 'SOCIAL_USER_REDIRECT_ON_ACCESS_TOKEN_FAILURE')
        if r:
            return redirect(r)
        raise

    # if the user is already authenticated, pass into the backend,
    #   so that the social identity can be associated to the user
    user = request.user if request.user.is_authenticated() else None

    user = auth(
        network=network, access_token=access_token,
        access_token_expires=access_token_expires, user=user)

    if user is not None:
        if user.is_active:
            login(request, user)
        else:
            # todo: handle this better
            raise SocialOauthDictFailed('User is not active')
    else:
        raise SocialOauthDictFailed('User not returned from authorization')

    django_social_user_post_callback.send(None, request=request)

    # redirect to the session value or the default redirect value
    r = request.session.get(SK_AUTH_REDIRECT) or getattr(
        options, 'SOCIAL_USER_REDIRECT_ON_AUTHENTICATION')
    if r:
        return redirect(r)
    raise



from django.contrib.auth.models import User
from django.db import models
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _


class SocialIdentity(models.Model):
    """
    A generic representation of an Identity on a social network.
    """
    # fields to make a unique id
    network = models.CharField(max_length=30,
        help_text='The name of the social network, should be defined on '
                  'extended GenericSocialUserBackend.')
    uid = models.CharField(max_length=255, help_text="The related network ID.")
    access_token = models.CharField(
        max_length=255, help_text="The access token for the social account.")
    access_token_expires = models.DateTimeField(null=True, blank=True)

    # fields to mirror user
    username = models.CharField(_('username'), max_length=255,
        help_text=_('Required. 255 characters or fewer. Letters, numbers and '
                    '@/./+/-/_ characters'))
    first_name = models.CharField(_('first name'), max_length=255, blank=True)
    last_name = models.CharField(_('last name'), max_length=255, blank=True)
    email = models.EmailField(_('e-mail address'), blank=True)

    # link to a Django user when possible
    user = models.ForeignKey(User, null=True,
        help_text="The user associated with this Social Identity; nullable.")

    # data for the related social network
    _data = models.TextField(db_column='data', blank=True)
    def set_data(self, data):
        self._data = simplejson.dumps(data)
    def get_data(self):
        return simplejson.loads(self._data)
    data = property(get_data, set_data)

    class Meta:
        unique_together = (('network', 'uid'),)



class SocialOauthDictFailed(ValueError):
    """
    Throw when backend is unable to fetch or parse data for get_oauth_dict.
    """
    pass

class SocialIdentityOwnedByAnotherUser(ValueError):
    """
    Thrown when the generic backend detects that the social identity
    is already owned by another user.
    """
    pass


from django.contrib.auth.backends import RemoteUserBackend
from django.contrib.auth.models import User
from django.utils import simplejson

from django_social_user.exceptions import SocialIdentityOwnedByAnotherUser
from django_social_user.models import SocialIdentity


class GenericSocialUserBackend(RemoteUserBackend):
    """
    Abstract backend class to be overwritten with network specific logic.
    """
    network = None

    def authenticate(self, network=None, access_token=None,
        access_token_expires=None, user=None):
        """
        Attempts to fetch and process remove social account data in order to
        authenticate in a generic way.
        """
        # authentication is not for this backend
        if not (self.network == network and access_token):
            raise TypeError

        # use oauth token to fetch the oauth object
        oauth_obj, uid = self.get_oauth_dict(access_token)

        email = self.get_email(oauth_obj)
        first_name = self.get_first_name(oauth_obj)
        last_name = self.get_last_name(oauth_obj)
        username = self.get_username(oauth_obj)

        social_identity, created = SocialIdentity.objects.get_or_create(
            network=network, uid=uid, defaults={
                'data': simplejson.dumps(oauth_obj),
                'email': email,
                'first_name': first_name,
                'last_name': last_name,
                'username': username,
            })

        # this social identity already exists
        if not created:
            # a user was passed in, make sure that it matches the user on
            #   the social identity
            if (user and social_identity.user_id and
                user.id != social_identity.user_id):
                raise SocialIdentityOwnedByAnotherUser

        # update the social identity with the provided user
        if not social_identity.user_id and user:
            social_identity.user = user
            social_identity.save()

        # check the access tokens
        if not hasattr(social_identity, 'access_token_expires'):
            # The account doesn't have an expiration property for its
            # access tokens. Simply compare the tokens and save.
            if social_identity.access_token != access_token:
                social_identity.access_token = access_token
                social_identity.save()
        else:
            # Only update the access token if there isn't currently
            # one or if the new one expires later than the current one
            # (or if the current one doesn't have an expiration date).
            new_token_expires_later = (
                social_identity.access_token_expires and
                access_token_expires and
                access_token_expires > social_identity.access_token_expires)
            if (not social_identity.access_token or
                not social_identity.access_token_expires or
                new_token_expires_later):
                social_identity.access_token = access_token
                social_identity.access_token_expires = access_token_expires
                social_identity.save()

        if not user:
            user, created = User.objects.get_or_create(username=username)
            if created:
                user.email = email
                user.first_name = first_name
                user.last_name = last_name
                user.save()

        return user

    def get_email(self, oauth_obj):
        """
        Get the email from the oauth object
        """
        raise NotImplementedError()

    def get_first_name(self, oauth_obj):
        """
        Get the first name from the oauth object
        """
        raise NotImplementedError()

    def get_last_name(self, oauth_obj):
        """
        Get the last name from the oauth object
        """
        raise NotImplementedError()

    def get_oauth_access_token(self, request, oauth_request_token):
        """
        Get the access token using the oauth request token and the data
        provided on the request. Should raise the appropriate exception
        if the data is invalid.
        """
        pass

    def get_oauth_authorization_url(self, oauth_request_token):
        """
        Create the authorization URL for redirect to social network.
        """
        raise NotImplementedError()

    def get_oauth_dict(self, access_token):
        """
        Uses the access token to find the oauth object,
        then returns it and the account uid:
        return (oauth_object, uid,)
        Raise SocialOauthDictFailed when response is invalid
        """
        raise NotImplementedError()

    def get_oauth_request_token(self):
        """
        Get the oauth request token from the network API.
        """
        raise NotImplementedError()

    def get_username(self, oauth_obj):
        """
        Get the username from the oauth object
        """
        raise NotImplementedError()



__author__ = 'mattesnider'

__version__ = (0, 1, 2)

registered_networks = {}


def register_backend(backend):
    """
    Registers a backend with the django social user system.
    """
    if not backend.network:
        raise Exception('network must be defined on custom backend')
    registered_networks[backend.network] = backend()



#!/usr/bin/env python
import os

ROOT_DIR = os.path.abspath(os.path.dirname(__file__))

# README is required for distribution, but README.md is required for github,
#   so create README temporarily
os.system('cp %s/README.md %s/README.txt' % (ROOT_DIR, ROOT_DIR))

sdict = dict(
    name = 'django-social-user',
    packages = ['django_social_user'],
    version='.'.join(map(str, __import__('django_social_user').__version__)),
    description = 'A generic system for interacting with remote APIs '
                  'that need to create Django users.',
    long_description=open('README.md').read(),
    url = 'https://github.com/mattsnider/Django-Social-User',
    author = 'Matt Snider',
    author_email = 'admin@mattsnider.com',
    maintainer = 'Matt Snider',
    maintainer_email = 'admin@mattsnider.com',
    keywords = ['social', 'user'],
    license = 'MIT',
    install_requires=['django>=1.3'],
    classifiers=[
        'Programming Language :: Python',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ]
)

from distutils.core import setup
setup(**sdict)

# cleanup README
os.remove('%s/README.txt' % ROOT_DIR)


from django.conf import settings

# if the user should be redirected to an error page on request token failure,
#   then define that redirect here, otherwise a 404 will be raised
SOCIAL_USER_REDIRECT_ON_REQUEST_TOKEN_FAILURE = getattr(settings,
    'SOCIAL_USER_REDIRECT_ON_REQUEST_TOKEN_FAILURE', '')

# if the user should be redirected to an error page on access token failure,
#   then define that redirect here, otherwise a 404 will be raised
SOCIAL_USER_REDIRECT_ON_ACCESS_TOKEN_FAILURE = getattr(settings,
    'SOCIAL_USER_REDIRECT_ON_ACCESS_TOKEN_FAILURE', '')

# this should be defined, if users should always be redirected to the
#   same page after authenticating. If either this value or the 'next' session
#   variable are not defined, then a 404 will be raised, because library
#   won't know where to redirect to.
SOCIAL_USER_REDIRECT_ON_AUTHENTICATION = getattr(settings,
    'SOCIAL_USER_REDIRECT_ON_AUTHENTICATION', '')